/* Hash table functions
   Taken from "The C Programming Language"
   by Kernighan & Ritchie
*/

#include <string.h>
#include <ctype.h>
#include "wrapmalloc.h"
#include "hash.h"

unsigned int hash (char *s, unsigned int hs)
{
	unsigned int hashval;
	
	if (!hs)
		return 0;

	for (hashval = 0; *s != '\0'; s++)
		hashval = (*s) + (31 * hashval);
	return hashval % hs;
}

HASH *lookupname (char *s, HASHTABLE *hs)
{
	HASH *current;
	unsigned int size = hs->size;

	for (current = (hs->start)[hash(s,size)]; current != NULL; current = current->next){
		if (!strcmp(s, current->name))
			return current;
	}
	return NULL;
}

HASH *installname (char *name, char *def, HASHTABLE *tab)
{
	HASH *curr;
	unsigned int hashval, size = tab->size;
	
	if ((curr = lookupname(name, tab)) == NULL){
		curr = (HASH *) MemAlloc (sizeof(HASH));
		if (curr == NULL || (curr->name = strdup(name)) == NULL)
			return NULL;
		hashval = hash(name, size);
		curr->next = (tab->start)[hashval];
		(tab->start)[hashval] = curr;
	}
	else
		MemFree((void *) curr->data);

	if ((curr->data = strdup (def)) == NULL)
		return NULL;
		
	return curr;
}

char * getdef (char * name, HASHTABLE *tab)
{
	HASH * tmp;
	
	tmp = lookupname (name, tab);
	if (tmp == NULL)
		return NULL;

	return tmp->data;
}

HASHTABLE * createhashtable (unsigned int size){
	HASHTABLE * tmp;
	int i;
	
	tmp = (HASHTABLE *) MemAlloc (sizeof(HASHTABLE));
	if (tmp){
		tmp->size = size;
		tmp->start = (HASH **) MemAlloc ((sizeof(HASH *)) * size +1);
		if (!tmp->start)
			return NULL;

		for(i = 0; i <= size; i++)
			(tmp->start)[i] = NULL;
		return tmp;
	}
	return NULL;
}
